package menu;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/** 
* This class manages a hierarchy of linked Page objects.  Each
* page has a related Displayable, and the commands associated
* with the Displayable enable the user to jump from Page to
* Page and perform various actions on those pages.
* <p>Some of the public fields are integer pageAction codes that
* are returned by the doCommandAction methods of the various Pages.
* The codes drive the cycling of Pages by this class.
* <p>Some of the public fields are Command objects.  These 
* commands can be used by the various Pages to attach Commands
* to their Displayables.  These are essentially singletons to 
* reduce memory usage.
* <p>$Id: PageMgr.java,v 1.4 2003/01/20 23:27:02 finson Exp $
*/
public class PageMgr implements CommandListener {
	/** 
	* Construct a new PageMgr.
	* @param m the QuitableMIDlet that we are part of.  We need
	* this object so that we can call quitApp
	* if the EXIT command is given.  We also use it to get
	* Display information.
	*/
	public PageMgr(QuitableMIDlet m) {
		midlet = m;
	}
	/**
	* Activate the device display by showing the root Page.
	* @param r the root Page of the hierarchy
	*/
	public void showRoot(Page r) {
		root = r;
		root.setPrevious(null);
		showPage(root);
	}
	/**
	* Activate the given Page.  If the Page implements the
	* Runnable interface, then call the start() method to get
	* the extra content thread going.
	* @param p the Page to show on the display.
	*/
	private void showPage(Page p) {
		currentPage = p;
		Displayable dsp = p.getDisplayable();
		dsp.setCommandListener(this);
		Display.getDisplay(midlet).setCurrent(dsp);
		if (p instanceof Runnable) {
			p.start();
		}
	}
	/**
	* Dispatch user commands and then adjust the display as needed.
	* Note to Page developers: the Page method doCommandAction must
	* return quickly because the system is blocked until we return.
	* @param c a Command object identifying the command.
	* @param d the Displayable on which this event has occurred
	*/
	public void commandAction(Command c, Displayable d) {
		Page p;
		int pageAction = currentPage.doCommandAction(c,d);
		switch (pageAction) {
			case ROOT: {
				showPage(root);
				break;
			}
			case BACK: 
			case CANCEL: {
				p = currentPage.getPrevious();
				if (p != null) showPage(p);
				break;
			}
			case EXIT: {
				try {
					midlet.quitApp();
				}
				catch (MIDletStateChangeException e) {
					e.printStackTrace();				// this should be an alert ...
				}
				break;
			}
			case LINK: {
				p = currentPage.getLinkedPage();
				p.setPrevious(currentPage);
				showPage(p);
				break;
			}
			case NONE: {
				break;
			}
			default: {
				throw new IllegalArgumentException(
				"Unrecognized pageAction in PageMgr: "+
				Integer.toString(pageAction));
			}
		}
	}
	
	//--------------------------------------------------------------------
	
	/** The Midlet that we are part of. */
	private QuitableMIDlet midlet;
	
	/** The Page that is currently on screen */
	private Page currentPage = null;
	
	/** The root Page of the hierarchy */
	private Page root;
	
	//--------------------------------------------------------------------
	
	/** Display the root page of the hierarchy */
	public static final int ROOT = 1;
	/** Display the previously displayed page */
	public static final int BACK = 2;
	/** Display the previously displayed page */
	public static final int CANCEL = 3;
	/** Exit the application entirely */
	public static final int EXIT = 4;
	/** 
	* Display a linked page.  
	* The next Page will be obtained with a call to getLinkedPage().
	*/
	public static final int LINK = 5;
	/** Continue to display the current page. */
	public static final int NONE = 6;
	
	//--------------------------------------------------------------------
	
	/** Return to previously displayed page. (p2) */
	public static final Command BACK_COMMAND = new Command("Back",Command.BACK,2);
	/** User cancels action proposal on displayed page. (p1) */
	public static final Command CANCEL_COMMAND = new Command("Cancel",Command.CANCEL,1);
	/** Exit the application entirely. (p2) */
	public static final Command EXIT_COMMAND = new Command("Exit",Command.EXIT,2);
	/** Display a help message. (p2) */
	public static final Command HELP_COMMAND = new Command("Help",Command.HELP,2);
	/** User confirms action proposal on displayed page. (p1) */
	public static final Command OK_COMMAND = new Command("OK",Command.OK,1);
	/** User requests termination of on-going action. (p2) */
	public static final Command STOP_COMMAND = new Command("Stop",Command.STOP,2);
	
}
		
